﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Input;
using ProjectorToolkit.Configuration;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Network;
using ProjectorToolkit.World;

namespace Client
{
    /// <summary>
    /// The client displays the world from any device in the same network from the chosen viewpoint.
    /// </summary>
    public partial class MainWindow : Window
    {
        private bool connecting = false;

        public MainWindow()
        {
            switch (App.Arguments.Length)
            {
                case 0:
                    InitializeComponent();
                    Closed += new EventHandler(WindowHelper.KillApplication);
                    KeyDown += new KeyEventHandler(WindowHelper.KillApplicationOnEscape);

                    // Parse configuration and add autocomplete display names
                    XmlConfiguration config = new XmlConfiguration("configuration.xml");
                    foreach (KeyValuePair<string, AbstractElement> kv in config.NamedElements
                        .Where(x => x.Value is OrthographicDisplay || x.Value is PerspectiveDisplay))
                        cb_name.Items.Add(kv.Key);

                    cb_name.Focus();
                    break;
                case 3:
                    try
                    {
                        string viewport = App.Arguments[0];
                        IPAddress ip = IPAddress.Parse(App.Arguments[1]);
                        int port = int.Parse(App.Arguments[2]);

                        try { StartClient(viewport, ip, port); }
                        catch (Exception) { FailedConnectingFromCommandLine(); }
                    }
                    catch (Exception)
                    {
                        FailedReadingCommandLineArguments();
                    }
                    break;
                default:
                    FailedReadingCommandLineArguments();
                    break;

            }
        }

        private void bt_start_Click(object sender, RoutedEventArgs e)
        {
            StartClientFromGui();
        }

        private void tb_name_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Return)
                StartClientFromGui();
        }

        private void StartClientFromGui()
        {
            if (!connecting)
            {
                connecting = true;
                try { StartClient(cb_name.Text, IPAddress.Parse(cb_ip.Text), int.Parse(tb_port.Text)); }
                catch (Exception) { FailedConnectingFromGUI(); }
                connecting = false;
            }
        }

        private void StartClient(string viewport, IPAddress ip, int port)
        {
            ViewportWindow mw = new ViewportWindow();
            ClientWorld world = new ClientWorld(mw.ViewportCanvas.Model3DGroup);
            mw.ViewportCanvas.World = world;

            BufferClient client = new BufferClient(ip, port);
            client.MessageReceived += world.WorldMessage;
            client.Buffering = false;

            this.Hide();
            mw.Show();
            mw.ViewportCanvas.Start(viewport);
        }

        #region Error Handling
        private void CommandlineError(string text)
        {
            System.Console.ForegroundColor = ConsoleColor.Red;
            System.Console.WriteLine("!!! Client Error !!!");
            System.Console.ForegroundColor = ConsoleColor.White;
            System.Console.WriteLine(text);
            WindowHelper.KillApplication();
        }
        private void FailedReadingCommandLineArguments()
        {
            CommandlineError(
                "Failed to read command line arguments!\n" +
                "Usage: Client.exe <viewport_name> <ip> <port>");
        }
        private void FailedConnectingFromCommandLine()
        {
            CommandlineError("Failed to connect with the server!");
        }
        private void FailedConnectingFromGUI()
        {
            MessageBox.Show("Failed to connect with the server!", "Client Error");
        }
        #endregion
    }
}