﻿using System.Collections.Generic;
using ProjectorToolkit.Controller;
using ProjectorToolkit.Elements;
using ProjectorToolkit.World;

namespace Layout
{
    public class LayoutHandler : MoveHandler
    {
        private Program program;
        private AbstractDisplay smartboard;

        public LayoutHandler(Button button, Program program, AbstractDisplay smartboard)
            : base(button)
        {
            this.program = program;
            this.smartboard = smartboard;
        }

        private static void Swap<T>(IList<T> list, int indexA, int indexB)
        {
            T tmp = list[indexA];
            list[indexA] = list[indexB];
            list[indexB] = tmp;
        }

        protected override void HandleMove(AbstractElement controller, AbstractElement target)
        {
            var targets = ServerWorld.Instance.PointingTargets(controller, target);
            ImageElement image = (ImageElement)target;

            if (targets.FindAll(x => x.Value == smartboard).Count > 0)
                image.Picture = image.Picture.Replace("Categories", "Details");
            else
                image.Picture = image.Picture.Replace("Details", "Categories");

            base.HandleMove(controller, target);
        }

        protected override void HandleReleased(AbstractElement controller, AbstractElement target)
        {
            var targets = ServerWorld.Instance.PointingTargets(controller, target);
            ImageElement image = (ImageElement)target;

            if (targets.FindAll(x => x.Value == smartboard).Count > 0)
            {
                var other = targets.Find(x => x.Value is ImageElement && x.Value != image);
                if (program.onBoardImages.Contains(image))
                {
                    // Swap both elements
                    if (!other.Equals(default(KeyValuePair<double, AbstractElement>)))
                        Swap(program.onBoardImages, program.onBoardImages.IndexOf(image), program.onBoardImages.IndexOf((ImageElement)other.Value));
                }
                else
                {
                    // Swap to right position
                    if (!other.Equals(default(KeyValuePair<double, AbstractElement>)))
                        program.onBoardImages.Insert(program.onBoardImages.IndexOf((ImageElement)other.Value), image);
                    else
                        program.onBoardImages.Add(image);
                    image.Picture = image.Picture.Replace("Categories", "Details");
                }
            }
            else
            {
                program.onBoardImages.Remove(image);
                image.Picture = image.Picture.Replace("Details", "Categories");
            }
            program.Layout();
        }
    }
}
