﻿using System.Windows;
using System.Windows.Media.Media3D;
using ProjectorToolkit;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Filter;
using ProjectorToolkit.Utilities;

namespace FilterDemo
{
    /// <summary>
    /// Filters the cursors position to allow comparison and parameter adjustments.
    /// </summary>
    public partial class MainWindow : Window
    {
        private const double WIDTH = 320;
        private const double HEIGHT = 240;
        private const double Z = -60;

        private AbstractElement controller;

        public MainWindow()
        {
            InitializeComponent();

            // Create a new environment
            IEnvironment env = new XmlEnvironment();

            // Add a bunch of images into the world (only for decoration
            env.World.Add(new ImageElement(new Point3D(-200, 1400, Z), WIDTH, HEIGHT, @"images\1.jpeg"));
            env.World.Add(new ImageElement(new Point3D(100, 1400, Z), WIDTH, HEIGHT, @"images\2.jpeg"));
            env.World.Add(new ImageElement(new Point3D(0, 1600, Z), WIDTH, HEIGHT, @"images\3.jpeg"));
            env.World.Add(new ImageElement(new Point3D(-100, 1200, Z), WIDTH, HEIGHT, @"images\4.jpeg"));

            // Get controller and register a cursor for it
            controller = env.World.Get("Projector");
            env.CursorDB.Register(controller);
        }

        private void NoFilter(object sender, RoutedEventArgs e)
        {
            controller.Filter = new NoFilter();
        }

        private void ElementJumpingFilter(object sender, RoutedEventArgs e)
        {
            controller.Filter = new ElementJumping();
        }

        private void OneEuroFilter(object sender, RoutedEventArgs e)
        {
            controller.Filter = new OneEuroFilter((double)OEF_MinCutoff.Value, (double)OEF_Beta.Value);
        }

        private void ConstantGridFilter(object sender, RoutedEventArgs e)
        {
            controller.Filter = new ConstantGridFilter((double)CGF_Constant.Value);
        }

        private void LinearGridFilter(object sender, RoutedEventArgs e)
        {
            controller.Filter = new LinearGridFilter((double)LGF_Factor.Value);
        }
    }
}
