﻿using System.Windows.Media.Media3D;
using ProjectorToolkit;
using ProjectorToolkit.Controller;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Relations;

namespace Two_Overlapping_Elements
{
    /// <summary>
    /// Two elements on the smartboard can be moved with the wii remote.
    /// They are changing their appearance when they overlap.
    /// </summary>
    public class Program
    {
        public const double Z = -60;

        public static void Main(string[] args)
        {
            new Program();
        }

        public Program()
        {
            // Create a new environment
            IEnvironment env = new XmlEnvironment();

            // Add two images into the world
            ImageElement element1 = new ImageElement(new Point3D(-250, 1400, Z), 320, 240, @"images\1.jpeg");
            ImageElement element2 = new ImageElement(new Point3D(250, 1400, Z), 320, 240, @"images\2.jpeg");
            env.World.Add(element1);
            env.World.Add(element2);

            // Use an overlapping relation
            OverlappingElementsRelation oer = new OverlappingElementsRelation(element1, element2);
            oer.OverlappingStarted += OnOverlappingStarted;
            oer.OverlappingEnded += OnOverlappingEnded;

            // Set controller and create handler to control the pictures
            WiiRemoteElement wiiRemote = new WiiRemoteElement();
            new MoveHandler(wiiRemote.ButtonA);
            env.Tracker.BindToTrackedObject(wiiRemote, "MarkerA");
            env.CursorDB.Register(wiiRemote);
        }

        private void OnOverlappingStarted(object sender, OverlappingElementsRelationEventArgs e)
        {
            ((ImageElement)e.Element1).Picture = @"images\3.jpeg";
            ((ImageElement)e.Element2).Picture = @"images\4.jpeg";
        }

        private void OnOverlappingEnded(object sender, OverlappingElementsRelationEventArgs e)
        {
            ((ImageElement)e.Element1).Picture = @"images\1.jpeg";
            ((ImageElement)e.Element2).Picture = @"images\2.jpeg";
        }
    }
}
