﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Media.Media3D;
using ProjectorToolkit.Elements;
using ProjectorToolkit.World;

namespace ProjectorToolkit.Controller
{
    sealed class CursorElement : ImageElement
    {
        private static string CURSOR_FOLDER = @"cursors\";
        public static string DEFAULT = CURSOR_FOLDER + "_DEFAULT_.png";

        protected override Type NetworkType
        {
            get
            {
                return typeof(ImageElement);
            }
        }

        public override Point3D? Intersection(Point3D origin, Vector3D direction)
        {
            return null;
        }

        public CursorElement() : base(new Point3D(0, 0, 0), 25, 25, DEFAULT) { }

        public void SetElement(AbstractElement element)
        {
            element.Updated += new UpdateEventHandler(PointerUpdate);
        }

        private void PointerUpdate(AbstractElement element)
        {
            AbstractElement target = ServerWorld.Instance.PointingTarget(element);
            if (target != null)
            {
                SetPose(
                    (Point3D)target.Intersection(element) + target.Direction,
                    (Vector3D.DotProduct(element.Direction, target.Direction) < 0) ? target.Direction : -target.Direction, 
                    target.Up);
            }
        }

        public void SetDefaultCursor()
        {
            Picture = DEFAULT;
        }

        public void SetHandlerCursor(Type handler)
        {
            string filename = CURSOR_FOLDER + handler.Name + ".png";
            if (File.Exists(filename))
                Picture = filename;
        }
    }

    /// <summary>
    /// The CursorDB creates a visual cursor for each registered element. 
    /// Handlers modify this cursor to indicate that an action is activated.
    /// Using CursorDB requires an "cursors" directory, where the application
    /// is created, that holds all necessary cursor images.
    /// </summary>
    public class CursorDB
    {
        private static Dictionary<AbstractElement, CursorElement> registration = new Dictionary<AbstractElement, CursorElement>();
        private static CursorDB instance;
        public static CursorDB Instance
        {
            get
            {
                if (instance == null)
                    instance = new CursorDB();
                return instance;
            }
        }
        private CursorDB() { }

        public void SetCursor(Type handler, AbstractElement pointer)
        {
            if (registration.ContainsKey(pointer))
            {
                registration[pointer].SetHandlerCursor(handler);
                ServerWorld.Instance.Focus(registration[pointer]);
            }
        }

        public void SetDefaultCursor(AbstractElement pointer)
        {
            if (registration.ContainsKey(pointer))
            {
                registration[pointer].SetDefaultCursor();
                ServerWorld.Instance.Focus(registration[pointer]);
            }
        }

        public void Register(AbstractElement pointer)
        {
            CursorElement cursor = new CursorElement();
            cursor.SetElement(pointer);
            registration[pointer] = cursor;
            ServerWorld.Instance.Add(cursor);
        }

        public void Unregister(AbstractElement pointer)
        {
            ServerWorld.Instance.Remove(registration[pointer]);
            registration.Remove(pointer);
        }
    }
}
