﻿using System.Windows.Media.Media3D;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkit.Controller
{
    /// <summary>
    /// This Handler moves the elements position and orientation according to
    /// the position and orientation of the controller.
    /// </summary>
    public class DirectManipulationHandler : AbstractHandler
    {
        private Point3D lastPosition;
        private Vector3D offset = new Vector3D(0, 0, 0);
        private Vector3D oldDirection;
        private Vector3D oldUp;

        public DirectManipulationHandler(Button button) : base(button) { }

        protected override bool CanHandleElement(AbstractElement element)
        {
            return element is RelativeElement;
        }

        protected override void HandlePressed(AbstractElement controller, AbstractElement target)
        {
            SetCurrentController(controller);
        }

        protected override void HandleMove(AbstractElement controller, AbstractElement target)
        {
            target.SetPose(
                target.Position + (controller.Position - lastPosition),
                Vector3DExtensions.SameRotation(target.Direction, oldDirection, controller.Direction),
                Vector3DExtensions.SameRotation(target.Up, oldUp, controller.Up));
            SetCurrentController(controller);
        }

        private void SetCurrentController(IPositionable controller)
        {
            lastPosition = controller.Position;
            oldDirection = controller.Direction;
            oldUp = controller.Up;
        }
    }
}
