﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ProjectorToolkit.Controller;

namespace ProjectorToolkit.Elements
{
    public class MouseElement : AbstractElement
    {
        private Control control;
        private AbstractDisplay viewport;
        private ProjectorToolkit.Controller.Button buttonLeft;
        private ProjectorToolkit.Controller.Button buttonMiddle;
        private ProjectorToolkit.Controller.Button buttonRight;
        #region Properties
        protected override Type NetworkType
        {
            get
            {
                return typeof(AbstractElement);
            }
        }
        public ProjectorToolkit.Controller.Button ButtonLeft
        {
            get { return buttonLeft; }
        }
        public ProjectorToolkit.Controller.Button ButtonMiddle
        {
            get { return buttonMiddle; }
        }
        public ProjectorToolkit.Controller.Button ButtonRight
        {
            get { return buttonRight; }
        }
        #endregion

        public MouseElement(Control control, AbstractDisplay viewport)
            : base()
        {
            this.viewport = viewport;
            this.control = control;

            control.MouseMove += new MouseEventHandler(MouseMove);
            control.MouseDown += new MouseButtonEventHandler(MouseButtonChange);
            control.MouseUp += new MouseButtonEventHandler(MouseButtonChange);

            buttonLeft = new ProjectorToolkit.Controller.Button(this);
            buttonMiddle = new ProjectorToolkit.Controller.Button(this);
            buttonRight = new ProjectorToolkit.Controller.Button(this);
        }

        private void MouseMove(object sender, MouseEventArgs e)
        {
            Point point = e.GetPosition(control);
            int x = (int)point.X;
            int y = (int)point.Y;
            if (viewport.PixelExists(x, y))
                SetPose(viewport.PixelOrigin(x, y), viewport.PixelDirection(x, y), Up);
        }

        private void AdjustButton(ProjectorToolkit.Controller.Button button, MouseButtonEventArgs e)
        {
            if (e.ButtonState == MouseButtonState.Pressed)
                button.State = ButtonState.Pressed;
            else
                button.State = ButtonState.Released;
        }

        private void MouseButtonChange(object sender, MouseButtonEventArgs e)
        {
            switch (e.ChangedButton)
            {
                case MouseButton.Left:
                    AdjustButton(buttonLeft, e);
                    break;
                case MouseButton.Middle:
                    AdjustButton(buttonMiddle, e);
                    break;
                case MouseButton.Right:
                    AdjustButton(buttonRight, e);
                    break;
            }
        }
    }
}
