﻿using System.Windows;
using System.Windows.Media.Media3D;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkit.Elements
{
    public static class PositionExtension
    {
        private static Vector3D Offset(Vector3D vector)
        {
            return vector * double.Epsilon;
        }

        /// <summary>
        /// Changes the position of the element to the given position on a wall.
        /// </summary>
        /// <param name="element">Element to reposition.</param>
        /// <param name="wall">Wall to position the element on.</param>
        /// <param name="point">Coordinate on the wall.</param>
        public static void PositionOn(this RelativeElement element, WallElement wall, Point point)
        {
            Plane plane = new Plane(wall.Position, wall.Direction);
            Point3D point3D = plane.AbsoluteCoordinateFor(point);
            Vector3D offset = Offset(plane.Normal);
            element.SetPose(point3D + offset, wall.Direction, wall.Up);
        }
        public static void PositionOn(this RelativeElement element, WallElement wall, double x, double y)
        {
            element.PositionOn(wall, new Point(x, y));
        }

        /// <summary>
        /// Changes the position of the element to the given pixel position on a display.
        /// </summary>
        /// <param name="element">Element to reposition.</param>
        /// <param name="display">Display to position the element on.</param>
        /// <param name="pixel">Pixel on the display (rounded to int).</param>
        public static void PositionOn(this RelativeElement element, OrthographicDisplay display, Point pixel)
        {
            element.PositionOn(display, (int)pixel.X, (int)pixel.Y);
        }
        public static void PositionOn(this RelativeElement element, OrthographicDisplay display, int xPixel, int yPixel)
        {
            double pixelWidth = display.Width / display.ResolutionX;
            Plane plane = new Plane(display.Position, display.Direction);

            Point pixelCoordinate = new Point(
                (xPixel - display.ResolutionX / 2) * pixelWidth, 
                -(yPixel - display.ResolutionY / 2) * pixelWidth);
            Point3D point = plane.AbsoluteCoordinateFor(pixelCoordinate);
            Vector3D offset =  Offset(-display.Direction);
            element.SetPose(point + offset, -display.Direction, display.Up);
        }
    }
}
