﻿using System;
using System.Diagnostics;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using GroupLab.iNetwork;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkit.Elements
{
    public class ImageElement : AbstractVirtualElement2D
    {
        private string picture = "";
        public string Picture
        {
            get { return picture; }
            set
            {
                if (!picture.Equals(value))
                {
                    picture = value;
                    UpdateNotification();
                }
            }
        }
        public override Polygon Corners
        {
            set
            {
                Debug.Assert(value.Length == 4);
                base.Corners = value;
            }
        }

        public ImageElement() : base() { }
        public ImageElement(double width, double height, string picture)
            : base(width, height)
        {
            Picture = picture;
        }
        public ImageElement(Point3D position, double width, double height, string picture)
            : base(position, width, height)
        {
            Picture = picture;
        }

        protected override Material Texture()
        {
            DiffuseMaterial material = new DiffuseMaterial();
            ImageBrush imageBrush = new ImageBrush();
            string path = System.Environment.CurrentDirectory + @"\" + Picture;
            try
            {
                imageBrush.ImageSource = new BitmapImage(new Uri(path));
            }
            catch (Exception)
            {
                throw new Exception("Picture does not exists: " + path);
            }
            material.Brush = imageBrush;
            return material;
        }

        #region Synchronization
        public override Message ToMessage(string messageName)
        {
            Message msg = base.ToMessage(messageName);
            msg.AddField("picture", Picture);
            return msg;
        }

        public override void UpdateElement(Message msg)
        {
            picture = msg.GetStringField("picture");
            base.UpdateElement(msg);
        }
        #endregion
    }
}
