﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ProjectorToolkit.Filter
{
    public interface IFilterFactory
    {
        AbstractFilter NewFilter();
    }

    public class NoFilterFactory : IFilterFactory
    {
        public AbstractFilter NewFilter()
        {
            return new NoFilter();
        }
    }
    public class ElementJumpingFactory : IFilterFactory
    {
        public AbstractFilter NewFilter()
        {
            return new ElementJumping();
        }
    }
    public class OneEuroFilterFactory : IFilterFactory
    {
        private double minCutoff;
        private double beta;

        public OneEuroFilterFactory(double minCutoff, double beta)
        {
            this.minCutoff = minCutoff;
            this.beta = beta;
        }

        public AbstractFilter NewFilter()
        {
            return new OneEuroFilter(minCutoff, beta);
        }
    }
    public class ConstantGridFilterFactory : IFilterFactory
    {
        private double thresholdDistance;

        public ConstantGridFilterFactory(double thresholdDistance)
        {
            this.thresholdDistance = thresholdDistance;
        }

        public AbstractFilter NewFilter()
        {
            return new ConstantGridFilter(thresholdDistance);
        }
    }
    public class LinearGridFilterFactory : IFilterFactory
    {
        private double linearFactor;

        public LinearGridFilterFactory(double linearFactor)
        {
            this.linearFactor = linearFactor;
        }

        public AbstractFilter NewFilter()
        {
            return new LinearGridFilter(linearFactor);
        }
    }
}
