﻿using System.Windows.Media.Media3D;
using ProjectorToolkit.Elements;
using ProjectorToolkit.World;

namespace ProjectorToolkit.Filter
{
    public abstract class GridFilter : AbstractFilter
    {
        private Point3D? lastHit;

        /// <summary>
        /// True if the filter needs to update, false otherwise.
        /// </summary>
        /// <param name="position">Position of the update</param>
        /// <param name="lastHit">The last hitpoint</param>
        /// <param name="totalDistance">The new hitpoint</param>
        /// <returns>True if update is needed, false otherwise.</returns>
        protected abstract bool ShouldUpdate(Point3D position, Point3D lastHit, Point3D newHit);

        public override void Filter(Point3D position, Vector3D direction, Vector3D up)
        {
            AbstractElement target = ServerWorld.Instance.PointingTarget(position, direction);
            if (target != null)
            {
                Point3D? hit = target.Intersection(position, direction);
                if (hit != null && (lastHit == null || ShouldUpdate(position, (Point3D)lastHit, (Point3D)hit)))
                {
                    lastHit = hit;
                    UpdateFilter(position, direction, up);
                }
            }
        }
    }
}
