﻿using System;
using System.Windows.Media.Media3D;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkit.Filter
{
    /// <summary>
    /// Updates only if the new target to last update distance
    /// is bigger than the grid size that is linear increasing
    /// with the distance to the target.
    /// </summary>
    public class LinearGridFilter : GridFilter
    {
        private double linearFactor;

        /// <summary>
        /// Create a bew relative grid filter.
        /// </summary>
        /// <param name="updatable">Object to update</param>
        /// <param name="linearFactor">Factor for multiplication with the old distance to calculate the maximum deviation.</param>
        public LinearGridFilter(double linearFactor)
        {
            this.linearFactor = linearFactor;
        }

        protected override bool ShouldUpdate(Point3D position, Point3D lastHit, Point3D newHit)
        {
            double totalDistance = Math.Abs(position.DistanceTo(lastHit));
            double distance = Math.Abs(lastHit.DistanceTo(newHit));
            return distance >= totalDistance * linearFactor;
        }
    }
}
