﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Media.Media3D;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkit.Relations
{
    public class DisplayArgs
    {
        public Polygon RelationArea;
        public List<Point[]> PixelOverlapping;
        public double NumberOfOverlappingPixels;
        public double Percentage;

        public DisplayArgs(AbstractDisplay display, Polygon area, Overlap overlapping)
        {
            RelationArea = area;
            PixelOverlapping = ToPixelOverlapping(display, overlapping);
            NumberOfOverlappingPixels = AreaPixelSum(PixelOverlapping);
            Percentage = NumberOfOverlappingPixels / display.TotalPixels;
        }

        private List<Point[]> ToPixelOverlapping(AbstractDisplay vp, Overlap global)
        {
            List<Point[]> result = new List<Point[]>();
            foreach (Polygon polygon in global.Overlaps)
            {
                Point3D[] points = global.PolygonToAbsolute(polygon);
                result.Add(vp.ConvertToPixels(points));
            }
            return result;
        }

        private double AreaPixelSum(List<Point[]> pixelOverlappings)
        {
            double result = 0;
            foreach (Point[] pixels in pixelOverlappings)
                result += (new Polygon(pixels)).Area;
            return result;
        }
    }

    public class ElementArgs
    {
        public double Percentage;

        public ElementArgs(IPositionable2D element, Overlap overlapping)
        {
            Percentage = overlapping.Area / element.Corners.Area;
        }
    }

    public class OverlappingRelationEventArgs : RelationEventArgs
    {
        public Overlap Overlapping;

        public OverlappingRelationEventArgs(Overlap overlapping)
        {
            Overlapping = overlapping;
        }
    }

    public abstract class AbstractOverlappingRelation<T> : AbstractRelation<T>
    {
    }
}
