﻿using System;
using ProjectorToolkit.Elements;

namespace ProjectorToolkit.Relations
{
    public enum RelationEventType { Start, Move, End }

    public class RelationEventArgs : EventArgs
    {
        public RelationEventType Type;
        public DateTime StartTime;
        public TimeSpan Duration;
        public int Count;
    }

    public abstract class AbstractRelation<T>
    {
        private bool isInUpdate = false;
        private DateTime startTime;
        private bool lastState = false;
        private int count = 0;
        
        protected abstract void Update(AbstractElement element);
        protected void NoRecursiveUpdate(AbstractElement element)
        {
            if (!isInUpdate)
            {
                isInUpdate = true;
                Update(element);
                isInUpdate = false;
            }
        }


        protected RelationEventType? SetRelationType(bool newState)
        {
            RelationEventType? type = null;
            if (!lastState && newState)
            {
                type = RelationEventType.Start;
                startTime = DateTime.Now;
                count++;
            }
            if (lastState && newState)
                type = RelationEventType.Move;
            if (lastState && !newState)
                type = RelationEventType.End;
            lastState = newState;
            return type;
        }

        protected void SetDefaultEventArgs(RelationEventArgs args, RelationEventType eventType)
        {
            args.Type = eventType;
            args.StartTime = startTime;
            args.Count = count;

            if (eventType == RelationEventType.Start)
                args.Duration = TimeSpan.FromSeconds(0);
            else
                args.Duration = DateTime.Now - startTime;
        }
    }
}
