﻿using System.Windows.Media.Media3D;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkit.Relations
{
    public class CrossingRelationEventArgs : RelationEventArgs
    {
        public AbstractElement Pointer;
        public AbstractElement CrossedElement;
        public Point3D EntryPoint;
        public Vector3D EntryPointFromCenter;
        public Point3D ExitPoint;
        public Vector3D ExitPointFromCenter;
        public double Distance;
    }

    /// <summary>
    /// A relation that triggers a callback when the pointer cuts the given element.
    /// Cutting is when the pointer enters and leaves an element under a given timespan.
    /// </summary>
    public sealed class CrossingRelation : AbstractRelation<CrossingRelationEventArgs>
    {
        public delegate void CrossingRelationEventHandler(object sender, CrossingRelationEventArgs e);
        public event CrossingRelationEventHandler Cutted = delegate { };
        private double maxDurationInSeconds;
        private PointingRelationEventArgs startPointingEventArgs;

        public CrossingRelation(AbstractElement pointer, AbstractElement cuttedElement, double maxDurationInSeconds = 1.0)
        {
            this.maxDurationInSeconds = maxDurationInSeconds;
            PointingRelation pr = new PointingRelation(pointer, cuttedElement);
            pr.Pointed += Update;
        }

        protected override void Update(AbstractElement element) { }
        protected void Update(object sender, PointingRelationEventArgs e)
        {
            SetRelationType(e.Type == RelationEventType.Start || e.Type == RelationEventType.Move);

            if (e.Type == RelationEventType.Start)
                startPointingEventArgs = e;
            if (e.Type == RelationEventType.End && e.Duration.Seconds <= maxDurationInSeconds)
            {
                Cutted(this, CreateEventArgs(e));
            }
        }

        private CrossingRelationEventArgs CreateEventArgs(PointingRelationEventArgs e)
        {
            CrossingRelationEventArgs args = new CrossingRelationEventArgs();
            args.Pointer = e.Pointer;
            args.CrossedElement = e.Target;
            args.EntryPoint = startPointingEventArgs.Intersection;
            args.EntryPointFromCenter = startPointingEventArgs.IntersectionFromCenter;
            args.ExitPoint = e.Intersection;
            args.ExitPointFromCenter = e.IntersectionFromCenter;
            args.Distance = args.EntryPoint.DistanceTo(args.ExitPoint);
            return args;
        }
    }
}
