﻿using System;
using System.Windows.Media.Media3D;

namespace ProjectorToolkit.Utilities
{
    public class Angle
    {
        private double angleInDegrees;
        #region Properties
        public double InDegrees
        {
            get { return angleInDegrees; }
        }
        public double InRadians
        {
            get { return DegreeToRadian(angleInDegrees); }
        }
        public bool IsNotZero
        {
            get { return !IsZero; }
        }
        public bool IsZero
        {
            get { return angleInDegrees == 0; }
        }
        #endregion

        private Angle(double degrees)
        {
            angleInDegrees = Normalize(degrees);
        }

        private static double Normalize(double degrees)
        {
            while (degrees < 0)
                degrees += 360;
            while (degrees >= 360)
                degrees -= 360;
            return degrees;
        }

        public static Angle FromDegrees(double degrees)
        {
            return new Angle(degrees);
        }

        public static Angle FromRadians(double radians)
        {
            return new Angle(RadianToDegree(radians));
        }

        /// <summary>
        /// Return the angle between two given vectors.
        /// </summary>
        /// <param name="vector1">First vector</param>
        /// <param name="vector2">Second vector</param>
        /// <returns>Angle between the two vectors.</returns>
        public static Angle Between(Vector3D vector1, Vector3D vector2)
        {
            return Angle.FromRadians(Math.Acos(Vector3D.DotProduct(vector1, vector2)));
        }

        private static double DegreeToRadian(double degree)
        {
            return degree * Math.PI / 180.0;
        }

        private static double RadianToDegree(double degree)
        {
            return degree * 180.0 / Math.PI;
        }
    }
}
