﻿using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using GroupLab.iNetwork;

namespace ProjectorToolkit.Utilities
{
    public static class MessageExtension
    {
        #region Vector3D
        public static void Add(this Message msg, string name, Vector3D vector)
        {
            msg.AddField(name + "_x", vector.X);
            msg.AddField(name + "_y", vector.Y);
            msg.AddField(name + "_z", vector.Z);
        }
        public static Vector3D GetVector3D(this Message msg, string name)
        {
            return new Vector3D(
                msg.GetDoubleField(name + "_x"),
                msg.GetDoubleField(name + "_y"),
                msg.GetDoubleField(name + "_z"));
        }
        #endregion

        #region Point3D
        public static void Add(this Message msg, string name, Point3D point)
        {
            msg.AddField(name + "_x", point.X);
            msg.AddField(name + "_y", point.Y);
            msg.AddField(name + "_z", point.Z);
        }
        public static Point3D GetPoint3D(this Message msg, string name)
        {
            return new Point3D(
                msg.GetDoubleField(name + "_x"),
                msg.GetDoubleField(name + "_y"),
                msg.GetDoubleField(name + "_z"));
        }
        #endregion

        #region Color
        public static void Add(this Message msg, string name, Color color)
        {
            msg.AddField(name + "_a", color.A);
            msg.AddField(name + "_r", color.R);
            msg.AddField(name + "_g", color.G);
            msg.AddField(name + "_b", color.B);
        }
        public static Color GetColor(this Message msg, string name)
        {
            return Color.FromArgb(
                msg.GetByteField(name + "_a"),
                msg.GetByteField(name + "_r"),
                msg.GetByteField(name + "_g"),
                msg.GetByteField(name + "_b"));
        }
        #endregion

        #region ConcurrentDictionary<string, double>
        public static void Add(this Message msg, string name, ConcurrentDictionary<string, double> dictionary)
        {
            msg.AddField(name + "_count", dictionary.Count);
            int i = 0;
            foreach (KeyValuePair<string, double> c in dictionary)
            {
                msg.AddField(name + "_key_" + i, c.Key);
                msg.AddField(name + "_value_" + i, c.Value);
                i++;
            }
        }
        public static ConcurrentDictionary<string, double> GetConcurrentDictionaryStringDouble(this Message msg, string name)
        {
            int count = msg.GetIntField(name + "_count");
            ConcurrentDictionary<string, double> result = new ConcurrentDictionary<string, double>();
            for (int i = 0; i < count; i++)
            {
                result.TryAdd(
                    msg.GetStringField(name + "_key_" + i),
                    msg.GetDoubleField(name + "_value_" + i));
            }
            return result;
        }
        #endregion

        #region BlockingCollection<string>
        public static void Add(this Message msg, string name, BlockingCollection<string> bag)
        {
            msg.AddField(name + "_count", bag.Count);
            int i = 0;
            foreach (string s in bag)
            {
                msg.AddField(name + "_" + i, s);
                i++;
            }
        }
        public static BlockingCollection<string> GetBlockingCollectionString(this Message msg, string name)
        {
            int count = msg.GetIntField(name + "_count");
            BlockingCollection<string> result = new BlockingCollection<string>();
            for (int i = 0; i < count; i++)
                result.Add(msg.GetStringField(name + "_" + i));
            return result;
        }
        #endregion
    }
}
