﻿using System;
using System.Windows.Media.Media3D;
using System.Windows.Threading;
using GroupLab.iNetwork;
using ProjectorToolkit.Elements;

namespace ProjectorToolkit.World
{
    public static class WorldMessageNames
    {
        public const string Update = "World";
        public const string Remove = "WorldRemove";
        public const string Focus = "WorldFocus";
    }
    public enum ElementState
    {
        New, Gone, Focus
    }
    public class UpdateEventEventArgs
    {
        public AbstractWorld World;
        public AbstractElement Element;
        public ElementState ElementState;
        public UpdateEventEventArgs(AbstractWorld world, AbstractElement element, ElementState state)
        {
            World = world;
            Element = element;
            ElementState = state;
        }
    }

    public class ClientWorld : AbstractWorld
    {
        public delegate void UpdateEventHandler(object sender, UpdateEventEventArgs e);
        public event UpdateEventHandler Updated = delegate { };
        private Model3DGroup modelGroup;

        public ClientWorld(Model3DGroup modelGroup)
        {
            this.modelGroup = modelGroup;
        }

        public void WorldMessage(object sender, Message msg)
        {
            string id;
            AbstractElement element;
            switch (msg.Name)
            {
                case WorldMessageNames.Update:
                    id = msg.GetStringField("id");
                    if (elements.ContainsKey(id))
                        elements[id].UpdateElement(msg);
                    else
                    {
                        modelGroup.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(delegate()
                        {
                            element = AbstractElement.CreateElement(msg, modelGroup);
                            RegisterElement(id, element);
                            Updated(this, new UpdateEventEventArgs(this, element, ElementState.New));
                        }));
                    }
                    break;

                case WorldMessageNames.Remove:
                    id = msg.GetStringField("id");
                    if (TryGetElement(id, out element))
                    {
                        UnregisterElement(id);
                        Updated(this, new UpdateEventEventArgs(this, element, ElementState.Gone));
                    }
                    break;

                case WorldMessageNames.Focus:
                    id = msg.GetStringField("id");
                    if (TryGetElement(id, out element))
                        Updated(this, new UpdateEventEventArgs(this, element, ElementState.Focus));
                    break;
            }
        }

        protected override void ElementMeshRefresh(AbstractElement element)
        {
            if (modelGroup != null)
                modelGroup.Dispatcher.Invoke(DispatcherPriority.Normal, new Action(delegate()
                {
                    element.UpdateMesh();
                }));
        }
    }
}
