﻿using System.Collections.Generic;
using System.Net;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using ProjectorToolkit.Configuration;
using ProjectorToolkit.Controller;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Network;
using ProjectorToolkit.Tracking;
using ProjectorToolkit.Utilities;
using ProjectorToolkit.World;

namespace ProjectorToolkit
{
    public class XmlEnvironment : IEnvironment
    {
#if LOCAL_USAGE
        SolidColorElement sce;
#endif
        private ITracker tracker;
        private ServerWorld world;
        private WorldServer server;
        #region Properties
        public ITracker Tracker
        {
            get { return tracker; }
        }
        public ServerWorld World
        {
            get { return world; }
        }
        public BufferServer Server
        {
            get { return server; }
        }
        public CursorDB CursorDB
        {
            get { return CursorDB.Instance; }
        }
        #endregion

        public XmlEnvironment()
        {
#if LOCAL_USAGE
            tracker = new NoTracker();
            server = new WorldServer(IPAddress.Parse("127.0.0.1"), 12345);
#else
            tracker = new ProximityTracker(2);
            server = new WorldServer(IPAddress.Parse("192.168.137.1"), 12345);
#endif
            world = new ServerWorld(server);

            // Load elements and bindings from configuration
            IConfiguration config = new XmlConfiguration("configuration.xml");

            foreach (KeyValuePair<string, AbstractElement> pair in config.NamedElements)
                world.Add(pair.Key, pair.Value);

            world.Add(config.UnnamedElements);

            foreach (TrackerBinding binding in config.TrackerBindings)
                tracker.Bind(binding);

#if LOCAL_USAGE
            // Create a debug halo for the projector
            sce = new SolidColorElement(new Point3D(20, 1438, -59), 0, 0, Colors.LightSalmon);
            world.Add(sce);
            config.NamedElements["Projector"].Updated += OnProjectorUpdate;
#endif
        }

#if LOCAL_USAGE
        private void OnProjectorUpdate(AbstractElement element)
        {
            Overlap overlap = Overlap.Of((AbstractDisplay)element, (IPositionable2D)(world.Get("Smartboard")));

            if (overlap != null)
                sce.Corners = overlap.Overlaps[0];
        }
#endif
    }
}
