﻿using System.Windows.Media.Media3D;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ProjectorToolkit.Elements;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkitTests.Elements
{
    [TestClass]
    public class MarkerElementTest
    {
        private void CheckVector(Vector3D expected, Vector3D actual, double delta = 0.001)
        {
            Assert.AreEqual(expected.X, actual.X, delta);
            Assert.AreEqual(expected.Y, actual.Y, delta);
            Assert.AreEqual(expected.Z, actual.Z, delta);
        }

        [TestMethod]
        public void ShouldResetPosition()
        {
            MarkerElement marker = new MarkerElement();
            marker.SetPosition(0, 0, 0);
            Assert.AreEqual(new Point3D(0, 0, 0), marker.Position);
            marker.SetPosition(1, 2, -3);
            Assert.AreEqual(new Point3D(1, 2, -3), marker.Position);
        }

        [TestMethod]
        public void Reorientate360ShouldDoNothing()
        {
            MarkerElement marker = new MarkerElement();
            marker.SetOrientation(new Vector3D(0, 0, -1), new Vector3D(0, 1, 0));
            marker.Rotate(new Vector3D(0, 1, 0), Angle.FromDegrees(360));
            CheckVector(new Vector3D(0, 0, -1), marker.Direction);
            CheckVector(new Vector3D(0, 1, 0), marker.Up);
        }

        [TestMethod]
        public void Reorientate180ShouldWork()
        {
            MarkerElement marker = new MarkerElement();
            marker.SetOrientation(new Vector3D(0, 0, -1), new Vector3D(0, 1, 0));
            marker.Rotate(new Vector3D(0, 1, 0), Angle.FromDegrees(180));
            CheckVector(new Vector3D(0, 0, 1), marker.Direction);
            CheckVector(new Vector3D(0, 1, 0), marker.Up);
        }
    }
}
