﻿using System.Windows.Media.Media3D;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkitTests
{
    [TestClass]
    public class Vector3DExtensionsTest
    {
        private Point3D axisPoint;
        private Vector3D axisDirection;
        private const double DELTA = 0.001;
        private Point3D inputCopy;
        private Point3D input;
        private readonly Angle ANGLE = Angle.FromDegrees(33.3);

        public Vector3DExtensionsTest()
        {
            input = new Point3D(10.1, 3, 5);
            inputCopy = new Point3D(10.1, 3, 5);
            axisPoint = new Point3D(13.4, 4.4, -43.1);
            axisDirection = new Vector3D(3.3, -4.2, 9.1);
        }

        public void Point3DAreEqual(Point3D p1, Point3D p2)
        {
            Assert.AreEqual(p1.X, p2.X, DELTA);
            Assert.AreEqual(p1.Y, p2.Y, DELTA);
            Assert.AreEqual(p1.Z, p2.Z, DELTA);
        }

        #region Input should not change
        [TestMethod]
        public void RotationAroundXAxisShouldNotChangeInput()
        {
            Point3D result = Vector3DExtensions.RotationAroundXAxis(input, ANGLE);
            Assert.AreEqual(inputCopy, input);
        }

        [TestMethod]
        public void RotationAroundYAxisShouldNotChangeInput()
        {
            Point3D result = Vector3DExtensions.RotationAroundYAxis(input, ANGLE);
            Assert.AreEqual(inputCopy, input);
        }

        [TestMethod]
        public void RotationAroundZAxisShouldNotChangeInput()
        {
            Point3D result = Vector3DExtensions.RotationAroundZAxis(input, ANGLE);
            Assert.AreEqual(inputCopy, input);
        }

        [TestMethod]
        public void RotationAroundArbitraryAxisShouldNotChangeInput()
        {
            Point3D result = Vector3DExtensions.RotationAroundArbitraryAxis(input, axisPoint, axisDirection, ANGLE);
            Assert.AreEqual(inputCopy, input);
        }
        #endregion

        #region Rotation should work
        [TestMethod]
        public void RotationAroundXAxisShouldWork()
        {
            Point3D result = Vector3DExtensions.RotationAroundXAxis(input, ANGLE);
            Point3DAreEqual(new Point3D(10.1, -0.2377, 5.8261), result);
        }

        [TestMethod]
        public void RotationAroundYAxisShouldWork()
        {
            Point3D result = Vector3DExtensions.RotationAroundYAxis(input, ANGLE);
            Point3DAreEqual(new Point3D(11.1868, 3, -1.3661), result);
        }

        [TestMethod]
        public void RotationAroundZAxisShouldWork()
        {
            Point3D result = Vector3DExtensions.RotationAroundZAxis(input, ANGLE);
            Point3DAreEqual(new Point3D(6.7946, 8.0526, 5), result);
        }

        [TestMethod]
        public void RotationAroundArbitraryAxisShouldWorkLikeBasicAxisRotations()
        {
            Point3D resultX1 = Vector3DExtensions.RotationAroundArbitraryAxis(input, new Point3D(0, 0, 0), new Vector3D(1, 0, 0), ANGLE);
            Point3D resultX2 = Vector3DExtensions.RotationAroundXAxis(input, ANGLE);
            Point3DAreEqual(resultX1, resultX2);

            Point3D resultY1 = Vector3DExtensions.RotationAroundArbitraryAxis(input, new Point3D(0, 0, 0), new Vector3D(0, 1, 0), ANGLE);
            Point3D resultY2 = Vector3DExtensions.RotationAroundYAxis(input, ANGLE);
            Point3DAreEqual(resultY1, resultY2);

            Point3D resultZ1 = Vector3DExtensions.RotationAroundArbitraryAxis(input, new Point3D(0, 0, 0), new Vector3D(0, 0, 1), ANGLE);
            Point3D resultZ2 = Vector3DExtensions.RotationAroundZAxis(input, ANGLE);
            Point3DAreEqual(resultZ1, resultZ2);
        }

        [TestMethod]
        public void RotationAroundArbitraryAxisThroughOriginShouldWork()
        {
            Point3D result = Vector3DExtensions.RotationAroundArbitraryAxis(input, new Point3D(0, 0, 0), axisDirection, ANGLE);
            Point3DAreEqual(new Point3D(6.2508, 6.0209, 7.7901), result);
        }

        [TestMethod]
        public void RotationAroundArbitraryAxisShouldWork()
        {
            Point3D result = Vector3DExtensions.RotationAroundArbitraryAxis(input, axisPoint, axisDirection, ANGLE);
            Point3DAreEqual(new Point3D(2.8991, -9.2716, 1.9475), result);
        }
        #endregion
    }
}
