﻿using System;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkitTests.Utilities
{
    [TestClass]
    public class PolygonHelperTest
    {
        private Polygon rectangle;
        private Polygon irregular;

        public PolygonHelperTest()
        {
            rectangle = new Polygon(new Point[] {
                new Point(10, 10),
                new Point(-1, 10),
                new Point(-1, -1),
                new Point(10, -1)
            });
            irregular = new Polygon(new Point[] {
                new Point(-3, -2),
                new Point(-1, 4),
                new Point(6, 1),
                new Point(3, 10),
                new Point(-4, 9)
            });
        }

        [TestMethod]
        public void ShouldChangePolygonDirection()
        {
            CheckReversed(rectangle, PolygonHelper.ChangeDirection(rectangle));
            CheckReversed(irregular, PolygonHelper.ChangeDirection(irregular));
        }

        [TestMethod]
        public void ChangeDirectionShouldBeReversible()
        {
            AreExactSame(rectangle, PolygonHelper.ChangeDirection(PolygonHelper.ChangeDirection(rectangle)));
            AreExactSame(irregular, PolygonHelper.ChangeDirection(PolygonHelper.ChangeDirection(irregular)));
        }

        public void CheckReversed(Polygon polygon, Polygon reversed)
        {
            Assert.AreEqual(polygon.Length, reversed.Length);
            for (int i = 0; i < polygon.Length; i++)
                Assert.AreEqual(polygon[i], reversed[reversed.Length - 1 - i]);
        }

        public void AreExactSame(Polygon polygon1, Polygon polygon2)
        {
            Assert.AreEqual(polygon1.Length, polygon2.Length);
            for (int i = 0; i < polygon1.Length; i++)
                Assert.AreEqual(polygon1[i], polygon2[i]);
        }
    }
}
