﻿using System;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ProjectorToolkit.Utilities;

namespace ProjectorToolkitTests.Utilities
{
    [TestClass]
    public class PolygonTest
    {
        private const double DELTA = 0.001;
        private Polygon rectangle;
        private Polygon irregular;

        public PolygonTest()
        {
            rectangle = new Polygon(new Point[] {
                new Point(10, 10),
                new Point(-1, 10),
                new Point(-1, -1),
                new Point(10, -1)
            });
            irregular = new Polygon(new Point[] {
                new Point(-3, -2),
                new Point(-1, 4),
                new Point(6, 1),
                new Point(3, 10),
                new Point(-4, 9)
            });
        }

        [TestMethod]
        public void ShouldContainCorners()
        {
            Assert.IsTrue(rectangle.Contains(rectangle[0]));
            Assert.IsTrue(rectangle.Contains(rectangle[1]));
            Assert.IsTrue(rectangle.Contains(rectangle[2]));
            Assert.IsTrue(rectangle.Contains(rectangle[3]));
        }

        [TestMethod]
        public void ShouldContainItself()
        {
            Assert.IsTrue(rectangle.Contains(rectangle));
        }
        
        [TestMethod]
        public void PolygonShouldBeAbleToCalculateArea()
        {
            Assert.AreEqual(121.0, rectangle.Area, 0.1);
            Assert.AreEqual(60.0, irregular.Area, 0.1);
        }

        [TestMethod]
        public void CheckIfPolygonIsConvex()
        {
            Assert.IsTrue(rectangle.IsConvex());
            Assert.IsFalse(irregular.IsConvex());
        }

        [TestMethod]
        public void IndexShouldCicleThroughPoints()
        {
            Assert.AreEqual(rectangle.Points[0], rectangle[-4]);
            Assert.AreEqual(rectangle.Points[0], rectangle[0]);
            Assert.AreEqual(rectangle.Points[0], rectangle[4]);

            Assert.AreEqual(rectangle.Points[1], rectangle[-3]);
            Assert.AreEqual(rectangle.Points[1], rectangle[1]);
            Assert.AreEqual(rectangle.Points[1], rectangle[5]);

            Assert.AreEqual(rectangle.Points[2], rectangle[10]);
            Assert.AreEqual(rectangle.Points[2], rectangle[-10]);
        }

        [TestMethod]
        public void ShouldCalculateCentroid()
        {
            Assert.AreEqual(new Point(4.5, 4.5), rectangle.Centroid());

            Point irrCentroid = irregular.Centroid();
            Assert.AreEqual(0.366, irrCentroid.X, DELTA);
            Assert.AreEqual(5.533, irrCentroid.Y, DELTA);
        }
    }
}
